@@ -646,7 +646,7 @@ netstatus_iface_get_inet4_details (NetstatusIface  *iface,
   strncpy (if_req.ifr_name, iface->priv->name, IF_NAMESIZE - 1);
   if_req.ifr_name [IF_NAMESIZE - 1] = '\0';
   if (mask && ioctl (fd, SIOCGIFNETMASK, &if_req) == 0)
-    *mask = g_strdup (inet_ntoa (((struct sockaddr_in *) &if_req.ifr_netmask)->sin_addr));
+    *mask = g_strdup (inet_ntoa (((struct sockaddr_in *) &if_req.ifr_addr)->sin_addr));
 
   close (fd);
 
@@ -925,6 +925,7 @@ netstatus_iface_get_hw_details (NetstatusIface  *iface,
 				char           **hw_addr)
 				
 {
+#ifdef SIOCGIFHWADDR
   static struct HwType *hw_type = NULL;
   struct ifreq          if_req;
   int                   fd;
@@ -973,6 +974,10 @@ netstatus_iface_get_hw_details (NetstatusIface  *iface,
     *hw_addr = hw_type->print_hw_addr (if_req.ifr_hwaddr.sa_data);
 
   return hw_type;
+
+#else /* !defined(SIOCGIFHWADDR) */
+  return NULL;
+#endif 
 }
 
 gboolean
@@ -1017,6 +1022,12 @@ netstatus_iface_get_device_details (NetstatusIface  *iface,
   return TRUE;
 }
 
+#if !defined(HAVE_SOCKADDR_SA_LEN)
+#define NETSTATUS_SA_LEN(saddr) (sizeof (struct sockaddr))
+#else
+#define NETSTATUS_SA_LEN(saddr) (MAX ((saddr)->sa_len, sizeof (struct sockaddr)))
+#endif /* HAVE_SOCKADDR_SA_LEN */
+
 /* Taken From R. Stevens Unix Network Programming Vol. 1.
  *
  * SIOCGIFCONF does not return an error on all systems if
@@ -1083,8 +1094,8 @@ netstatus_list_interface_names (GError **error)
   struct ifconf *if_conf;
   GList         *interfaces;
   GList         *loopbacks;
+  char          *p;
   int            fd;
-  int            i;
   
   if ((fd = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
     {
@@ -1105,12 +1116,14 @@ netstatus_list_interface_names (GError **error)
   interfaces = NULL;
   loopbacks  = NULL;
 
-  for (i = 0; i < if_conf->ifc_len / sizeof (struct ifreq); i++)
+  for (p = if_conf->ifc_buf; p < if_conf->ifc_buf + if_conf->ifc_len;)
     {
-      struct ifreq if_req = if_conf->ifc_req [i];
-      gboolean     loopback = FALSE;
+      struct ifreq *if_req = (struct ifreq *) p;
+      gboolean      loopback = FALSE;
 
-      if (ioctl (fd, SIOCGIFFLAGS, &if_req) < 0)
+      p += sizeof (if_req->ifr_name) + NETSTATUS_SA_LEN (&if_req->ifr_addr);
+
+      if (ioctl (fd, SIOCGIFFLAGS, if_req) < 0)
 	{
 	  if (error)
 	    *error = g_error_new (NETSTATUS_ERROR,
@@ -1120,13 +1133,15 @@ netstatus_list_interface_names (GError **error)
 	}
       else
 	{
-	  loopback = (if_req.ifr_flags & IFF_LOOPBACK);
+	  loopback = (if_req->ifr_flags & IFF_LOOPBACK);
 	}
 
       if (!loopback)
-	interfaces = g_list_prepend (interfaces, g_strdup (if_req.ifr_name));
+	interfaces = netstatus_list_insert_unique (interfaces,
+						   g_strdup (if_req->ifr_name));
       else
-	loopbacks  = g_list_prepend (loopbacks,  g_strdup (if_req.ifr_name));
+	loopbacks  = netstatus_list_insert_unique (loopbacks,
+						   g_strdup (if_req->ifr_name));
     }
 
   interfaces = g_list_concat (interfaces, loopbacks);
